@@ -5,11 +5,14 @@ __author__ = 'Jens-Kristian Krogager'
 import numpy as np
 import matplotlib.pyplot as plt
 import copy
+import warnings
 
+from astropy.io import fits
 from lmfit import Parameters, Minimizer
 
 from . import Asplund
 from .components import Component
+from .fits_input import load_fits_spectrum, FormatError, MultipleSpectraWarning
 from . import hdf5_save
 from . import line_complexes
 from .line_complexes import fine_structure_complexes
@@ -23,6 +26,23 @@ from .voigt import evaluate_profile, evaluate_continuum
 
 myfloat = np.float64
 
+def air2vac(air):
+    """
+    Air to vacuum conversion from Bengt Edlén 1953,
+    Journal of the Optical Society of America, Vol. 43, Issue 5, pp. 339-344.
+    """
+    if np.min(air) < 2000.:
+        raise ValueError("Input wavelength below valid range!")
+    air = np.array(air)
+    ij = (np.array(air) >= 2000)
+    out = np.array(air).copy()
+    s2 = (1.e4/air)**2
+    fact = 1.0 + 6.4328e-5 + 2.94981e-2/(146.0 - s2) + 2.5540e-4/(41.0 - s2)
+    # Alternative solution from VALD:
+    # fact = 1 + 8.336624e-5 + 0.02408927 / (130.106592 - s2) + 1.599740895e-4 / (38.925688 - s2),
+    out[ij] = air[ij]*fact[ij]
+    return out
+
 
 def mask_vel(dataset, line_tag, v1, v2):
     regions_of_line = dataset.find_line(line_tag)
@@ -143,6 +163,7 @@ class DataSet(object):
         # container for data chunks to be fitted
         # data should be added by calling method 'add_data'
         self.data = list()
+        self.data_filenames = list()
 
         self.verbose = True
 
@@ -184,7 +205,7 @@ class DataSet(object):
         """Returns the name of the DataSet."""
         return self.name
 
-    def add_spectrum(self, filename, res, normalized=False, mask=None, continuum=None, nsub=1):
+    def add_spectrum(self, filename, res, airORvac='vac', normalized=False, mask_array=None, mask=None, use_mask=True, continuum=None, nsub=1, verbose=False, ext=None):
         """
         Add spectral data from ASCII file (three or four columns accepted).
         This is a wrapper of the method `add_data`.
@@ -203,38 +224,111 @@ class DataSet(object):
             line-spread function for the given spectrum.
             See details in the documentation.
 
+        airORvac : string  {'vac' or 'air'}
+            Defines whether the input wavelengths are in 'air' or 'vacuum' units. If given as 'air'
+            the wavelengths will be converted to 'vacuum'.
+
         normalized : bool   [default = False]
             If the input spectrum is normalized this should be given as True
             in order to skip normalization steps.
 
-        mask : array, shape (n)
+        mask_array : array, shape (n)
             Boolean/int array defining the fitting regions.
             Only pixels with mask=True/1 will be included in the fit.
 
+        mask : Deprecated, use `mask_array`
+
+        use_mask : bool   [default = True]
+            If False, do not pass the mask array to the dataset. This may be useful if the mask defined in the spectrum
+            is not appropriate for the fitting purposes.
+
         continuum : array, shape (n)
             Continuum spectrum. The input spectrum will be normalized using
             this continuum spectrum.
 
-        nsub : integer
+        nsub : integer   [default = 1]
             Kernel subsampling factor relative to the data.
             This is only used if the resolution is given as a LSF file.
+
+        verbose : bool   [default = False]
+            Print status messages.
+
+        ext : int or string
+            Index or name of the extension in the HDU List. Only used if the input is a FITS file.
         """
+        if mask is not None:
+            warnings.warn("The keyword mask is deprecated. Use 'mask_array' instead", DeprecationWarning)
+            mask_array = mask
 
-        spectrum = np.loadtxt(filename)
-        wl = spectrum[:, 0]
-        flux = spectrum[:, 1]
-        err = spectrum[:, 2]
-        if spectrum.shape[1] == 4 and mask is None:
-            mask = spectrum[:, 3]
+        file_type = filename.split('.')[-1]
+        if file_type.lower() in ['fits', 'fit']:
+            with warnings.catch_warnings(record=True) as warning_list:
+                try:
+                    wl, spec, err, mask, _ = load_fits_spectrum(filename, ext=ext)
+                    has_multiSpecWarning = any([w.category is MultipleSpectraWarning for w in warning_list])
+                    if has_multiSpecWarning:
+                        # Show warning that there are multiple spectra
+                        print(term.red)
+                        print("\n [WARNING] - Several data tables were detected. The first extension was used.")
+                        print("")
+                        fits.info(filename)
+                        print(term.reset)
+                        print("")
 
-        if continuum is not None:
-            flux = flux/continuum
+                except FormatError as error_msg:
+                    print(error_msg)
+                    print("")
+                    fits.info(filename)
+                    print("")
+                    fits_data = fits.getdata(filename)
+                    if isinstance(fits_data, fits.FITS_rec):
+                        # Show the Table Column definitions:
+                        print(fits_data.columns)
+                        raise
+
+        else:
+            data = np.loadtxt(filename)
+            if data.shape[1] == 3:
+                wl, spec, err = data.T
+                mask = np.ones_like(wl, dtype=bool)
+            elif data.shape[1] == 4:
+                wl, spec, err, mask = data.T
+                mask = mask.astype(bool)
+            elif data.shape[1] > 4:
+                wl = data[:, 0]
+                spec = data[:, 1]
+                err = data[:, 2]
+                mask = data[:, 3]
+                mask = mask.astype(bool)
+            else:
+                print(" [ERROR] - Not enough columns to load wavelength, flux and error")
+                print("           Check file format, must be three or more columns separated by blank space")
+                print("")
+                raise FormatError
+
+        if airORvac == 'air':
+            if verbose:
+                print(" Converting wavelength from air to vacuum.")
+            wl = air2vac(wl)
+
+        if continuum is not None and len(continuum) == len(spec):
+            spec = spec/continuum
             err = err/continuum
             normalized = True
 
-        self.add_data(wl, flux, res, err=err, normalized=normalized, mask=mask, nsub=nsub)
+        if mask_array is not None:
+            if len(mask_array) == len(spec):
+                mask = mask_array
+            else:
+                print("Wrong dimensions of the Input Mask: got %i pixels, spectrum has %i pixels" % (len(mask_array), len(spec)))
 
-    def add_data(self, wl, flux, res, err=None, normalized=False, mask=None, nsub=1):
+        if use_mask:
+            self.add_data(wl, spec, res, err=err, normalized=normalized, mask=mask, nsub=nsub)
+        else:
+            self.add_data(wl, spec, res, err=err, normalized=normalized, nsub=nsub)
+        self.data_filenames.append(filename)
+
+    def add_data(self, wl, flux, res, err=None, normalized=False, mask=None, nsub=1, filename=''):
         """
         Add spectral data to the DataSet. This will be used to define fitting regions.
 
@@ -268,7 +362,13 @@ class DataSet(object):
         nsub : integer
             Kernel subsampling factor relative to the data.
             This is only used if the resolution is given as a LSF file.
+
+        filename : string
+            The filename from which the data originated. Optional but highly recommended.
+            Alternatively, use the method :meth:`DataSet.add_spectrum
+            <VoigtFit.DataSet.add_spectrum>`.
         """
+
         mask_warning = """
         All pixels in the spectrum have been masked out.
         Pixels to *include* in the fit should have value = 1.
@@ -300,6 +400,7 @@ class DataSet(object):
                           'error': err, 'res': res,
                           'norm': normalized, 'specID': specid,
                           'mask': mask, 'nsub': nsub})
+        self.data_filenames.append(filename)
 
     def reset_region(self, reg):
         """Reset the data in a given :class:`regions.Region` to use the raw input data."""
@@ -998,6 +1099,9 @@ class DataSet(object):
         is assumed to be unresolved, i.e., taken from the resolution.
         """
         regions_of_line = self.find_line(line_tag)
+        if len(regions_of_line) > 1:
+            print("\n Note -- The given line (%s) is defined in more than one spectral region")
+            print("        All regions will be shown one by one for interactive definition...\n")
         for region in regions_of_line:
             wl, flux, err, mask = region.unpack()
             plt.close('all')
@@ -1036,41 +1140,64 @@ class DataSet(object):
             if region.normalized:
                 c_level = 1.
             else:
-                ax.set_title("Click to Mark Continuum Level...")
+                ax.set_title("Click to Mark Average Continuum Level...\n(1 click)")
                 cont_level_point = plt.ginput(1, 30.)
                 c_level = cont_level_point[0][1]
                 ax.axhline(c_level, color='0.3', ls=':')
 
-            ax.set_title("Mark central components for %s, finish with [enter]" % line.element)
+            ax.set_title("Mark peak absorption for components of %s\nFinish with 'enter'" % line.tag)
             ax.set_xlabel(x_label)
             if region.normalized:
                 ax.set_ylabel(u"Normalized Flux")
             else:
                 ax.set_ylabel(u"Flux")
-            comps = plt.ginput(-1, 60)
-            num = 0
+            plt.draw()
+            plt.tight_layout()
+
             # Assume that components are unresolved:
             b = region.kernel_fwhm / 2.35482
+            comp_lines = list()
             comp_list = list()
-            for x0, y0 in comps:
-                if velocity:
-                    z0 = self.redshift + x0/299792.458 * (self.redshift + 1.)
+            success = False
+            while success is False:
+                comps = plt.ginput(-1, 60)
+                for x0, y0 in comps:
+                    if velocity:
+                        z0 = self.redshift + x0/299792.458 * (self.redshift + 1.)
+                    else:
+                        z0 = x0/line.l0 - 1.
+                    # Calculate logN from peak depth:
+                    y0 = max(y0/c_level, 0.01)
+                    logN = np.log10(-b * np.log(y0) / (1.4983e-15 * line.l0 * line.f))
+
+                    vline = ax.axvline(x0, color='darkblue', alpha=0.8)
+                    comp_lines.append(vline)
+                    comp_list.append([z0, b, logN])
+                plt.draw()
+
+                print("Are the shown components correct?")
+                ax.set_title("Are the shown components correct?")
+                plt.draw()
+                answer = input("[YES/no]  ")
+                if answer.lower() in ['', 'yes', 'y']:
+                    success = True
                 else:
-                    z0 = x0/line.l0 - 1.
-                # Calculate logN from peak depth:
-                y0 = max(y0/c_level, 0.01)
-                logN = np.log10(-b * np.log(y0) / (1.4983e-15 * line.l0 * line.f))
-                print("Component %i:  z = %.6f   log(N) = %.2f" % (num, z0, logN))
-                ax.axvline(x0, color='darkblue', alpha=0.8)
-                comp_list.append([z0, b, logN])
-                num += 1
-            plt.draw()
+                    comp_list = list()
+                    for vline in comp_lines:
+                        vline.remove()
+                    comp_lines = list()
+                    ax.set_title("Mark peak absorption for components of %s\nFinish with 'enter'" % line.tag)
+                    plt.draw()
 
             if len(comp_list) > 0:
+                print("Defining following components:")
+                print("(the lines below can be copied directly to the input file)\n")
                 for z, b, logN in comp_list:
-                    self.add_component(line.element, z, b, logN)
+                    print("component %s  z=%.6f  b=%.1f  logN=%.2f" % (line.ion, z, b, logN))
+                    self.add_component(line.ion, z, b, logN)
             else:
-                pass
+                print("No components were defined. Are you sure you want to continue?")
+
 
     def delete_component(self, ion, index):
         """Remove component of the given `ion` with the given `index`."""
@@ -1876,15 +2003,15 @@ class DataSet(object):
 
             if verbose:
                 output.print_results(self, self.best_fit, velocity=True)
-                if self.cheb_order >= 0:
-                    output.print_cont_parameters(self)
+                # if self.cheb_order >= 0:
+                #     output.print_cont_parameters(self)
 
         chi2 = popt.chisqr
         return popt, chi2
 
     def plot_fit(self, rebin=1, fontsize=12, xmin=None, xmax=None, max_rows=4,
                  ymin=None, ymax=None, filename=None,
-                 subsample_profile=1, npad=50, loc='left',
+                 subsample_profile=10, npad=50, loc='left',
                  highlight_props=None, residuals=True, norm_resid=False,
                  default_props={}, element_props={}, legend=True,
                  label_all_ions=False, xunit='vel',
