@@ -49,7 +49,9 @@ def main():
     print(r"                \/                     ")
     print(r"")
     print(r"")
-    print(r" Loaded Solar abundances from Asplund et al. 2009  (photospheric)")
+    print(r" Loaded Solar abundances from Asplund et al. (2009)")
+    print(r" Using recommendations from Lodders et al. (2009)")
+    print(r" regarding the source: photospheric, meteoritic or their average.")
     print(r"")
 
     descr = """VoigtFit Absorption Line Fitting.
@@ -549,8 +551,8 @@ def main():
         # print Thermal Model Parameters
         output.print_T_model_pars(dataset, thermal_model)
 
-    if dataset.cheb_order >= 0:
-        dataset.print_cont_parameters()
+    # if dataset.cheb_order >= 0:
+    #     dataset.print_cont_parameters()
 
     # print metallicity
     logNHI = parameters['logNHI']
@@ -580,7 +582,7 @@ def main():
         rebin = parameters['fit_options']['rebin']
     else:
         rebin = 1
-    dataset.plot_fit(filename=filename, rebin=rebin, subsample_profile=3)
+    dataset.plot_fit(filename=filename, rebin=rebin)
     output.save_parameters_to_file(dataset, filename+'.fit')
     output.save_cont_parameters_to_file(dataset, filename+'.cont')
     output.save_fit_regions(dataset, filename+'.reg',
